# -*- coding: utf-8 -*-
from yookassa.domain.common.context import Context


class DataContext(Context):
    """
    Constants representing context data types. Available values are:

    * yookassa.domain.common.DataContext.REQUEST
    * yookassa.domain.common.DataContext.RESPONSE
    """
    REQUEST = 'request'
    RESPONSE = 'response'



# -*- coding: utf-8 -*-
from datetime import datetime, timedelta

from yookassa.domain.common.payment_method_type import PaymentMethodType
from yookassa.domain.common.request_object import RequestObject
from yookassa.domain.models.airline import Airline
from yookassa.domain.models.amount import Amount
from yookassa.domain.models.confirmation.confirmation import Confirmation
from yookassa.domain.models.confirmation.confirmation_factory import ConfirmationFactory
from yookassa.domain.models.deal import PaymentDealInfo
from yookassa.domain.models.payment_data.payment_data import PaymentData
from yookassa.domain.models.payment_data.payment_data_factory import PaymentDataFactory
from yookassa.domain.models.receipt import Receipt
from yookassa.domain.models.recipient import Recipient
from yookassa.domain.models.transfer import Transfer

DESCRIPTION_MAX_LENGTH = 128
MERCHANT_CUSTOMER_MAX_LENGTH = 200


class PaymentRequest(RequestObject):
    """
    Class representing PaymentRequest values enum
    """

    __amount = None

    __description = None

    __receipt = None

    __recipient = None

    __payment_token = None

    __payment_method_id = None

    __payment_method_data = None

    __confirmation = None

    __save_payment_method = None

    __capture = None

    __client_ip = None

    __metadata = None

    __airline = None

    __transfers = []

    __deal = None

    __merchant_customer_id = None

    @property
    def amount(self):
        return self.__amount

    @amount.setter
    def amount(self, value):
        if isinstance(value, dict):
            self.__amount = Amount(value)
        elif isinstance(value, Amount):
            self.__amount = value
        else:
            raise TypeError('Invalid amount value type')

    @property
    def description(self):
        return self.__description

    @description.setter
    def description(self, value):
        cast_value = str(value)
        if cast_value:
            if len(cast_value) <= DESCRIPTION_MAX_LENGTH:
                self.__description = cast_value
            else:
                raise ValueError('The value of the description parameter is too long. Max length is {}'.format(DESCRIPTION_MAX_LENGTH))
        else:
            raise ValueError('Invalid description value')

    @property
    def receipt(self):
        return self.__receipt

    @receipt.setter
    def receipt(self, value):
        if isinstance(value, dict):
            self.__receipt = Receipt(value)
        elif isinstance(value, Receipt):
            self.__receipt = value
        else:
            raise TypeError('Invalid receipt_request value type')

    @property
    def recipient(self):
        return self.__recipient

    @recipient.setter
    def recipient(self, value):
        if isinstance(value, dict):
            self.__recipient = Recipient(value)
        elif isinstance(value, Recipient):
            self.__recipient = value
        else:
            raise TypeError('Invalid recipient value type')

    @property
    def payment_token(self):
        return self.__payment_token

    @payment_token.setter
    def payment_token(self, value):
        cast_value = str(value)
        if cast_value:
            self.__payment_token = cast_value
        else:
            raise TypeError('Invalid payment_token value')

    @property
    def payment_method_id(self):
        return self.__payment_method_id

    @payment_method_id.setter
    def payment_method_id(self, value):
        cast_value = str(value)
        if cast_value:
            self.__payment_method_id = cast_value

    @property
    def payment_method_data(self):
        return self.__payment_method_data

    @payment_method_data.setter
    def payment_method_data(self, value):
        if isinstance(value, dict):
            self.__payment_method_data = PaymentDataFactory().create(value, self.context())
        elif isinstance(value, PaymentData):
            self.__payment_method_data = value
        else:
            raise TypeError('Invalid payment_method_data type')

    @property
    def confirmation(self):
        return self.__confirmation

    @confirmation.setter
    def confirmation(self, value):
        if isinstance(value, dict):
            self.__confirmation = ConfirmationFactory().create(value, self.context())
        elif isinstance(value, Confirmation):
            self.__confirmation = value
        else:
            raise TypeError('Invalid confirmation type')

    @property
    def save_payment_method(self):
        return self.__save_payment_method

    @save_payment_method.setter
    def save_payment_method(self, value):
        self.__save_payment_method = bool(value)

    @property
    def capture(self):
        return self.__capture

    @capture.setter
    def capture(self, value):
        self.__capture = bool(value)

    @property
    def client_ip(self):
        return self.__client_ip

    @client_ip.setter
    def client_ip(self, value):
        self.__client_ip = str(value)

    @property
    def airline(self):
        return self.__airline

    @airline.setter
    def airline(self, value):
        if isinstance(value, dict):
            self.__airline = Airline(value)
        elif isinstance(value, Airline):
            self.__airline = value
        else:
            raise TypeError('Invalid airline type')

    @property
    def metadata(self):
        return self.__metadata

    @metadata.setter
    def metadata(self, value):
        if type(value) is dict:
            self.__metadata = value

    @property
    def transfers(self):
        return self.__transfers

    @transfers.setter
    def transfers(self, value):
        if isinstance(value, list):
            self.__transfers = [Transfer(item) for item in value]
        elif value is None:
            self.__transfers = []
        else:
            raise TypeError('Invalid transfers data type in payment_request.transfers')

    @property
    def deal(self):
        return self.__deal

    @deal.setter
    def deal(self, value):
        if isinstance(value, dict):
            self.__deal = PaymentDealInfo(value)
        elif isinstance(value, PaymentDealInfo):
            self.__deal = value
        else:
            raise TypeError('Invalid deal type')

    @property
    def merchant_customer_id(self):
        return self.__merchant_customer_id

    @merchant_customer_id.setter
    def merchant_customer_id(self, value):
        cast_value = str(value)
        if cast_value:
            if len(cast_value) <= MERCHANT_CUSTOMER_MAX_LENGTH:
                self.__merchant_customer_id = cast_value
            else:
                raise ValueError('The value of the merchant_customer_id parameter is too long. Max length is {}'
                                 .format(MERCHANT_CUSTOMER_MAX_LENGTH))
        else:
            raise ValueError('Invalid description value')

    def validate(self):
        amount = self.amount
        if amount is None:
            self.__set_validation_error('Payment amount not specified')

        if amount.value <= 0.0:
            self.__set_validation_error('Invalid payment amount value: ' + str(amount.value))

        if self.receipt is not None and self.receipt.has_items:
            email = self.receipt.email
            phone = self.receipt.phone
            if not email and not phone:
                self.__set_validation_error('Both email and phone values are empty in receipt')

            if self.receipt.tax_system_code is None:
                for item in self.receipt.items:
                    if item.vat_code is None:
                        self.__set_validation_error('Item vat_code and receipt tax_system_id not specified')

        if self.payment_token:
            if self.payment_method_id:
                self.__set_validation_error('Both payment_token and payment_method_id values are specified')

            if self.payment_method_data:
                self.__set_validation_error('Both payment_token and payment_data values are specified')

        elif self.payment_method_id:
            if self.payment_method_data:
                self.__set_validation_error('Both payment_method_id and payment_data values are specified')

        if self.payment_method_data and self.payment_method_data.type == PaymentMethodType.BANK_CARD \
                and self.payment_method_data.card is not None:
            # Get current date with an offset.
            # Why? Because expiration is relative to the transaction
            # processor's timezone, which we don't know. 1 day allowance will
            # prevent any sorts of false negatives, leaving a small
            # less-than-day window for the unlikely false positives.
            # Account for GMT-12 to GMT+14 difference
            # + DST: https://www.timeanddate.com/time/dst/
            date_now = datetime.now() - timedelta(hours=26 + 1)

            # Expiry date is "valid thru", meaning it is valid until the last
            # second of the last day of the month in the date.
            date_expiry = datetime(
                year=int(self.payment_method_data.card.expiry_year),
                month=int(self.payment_method_data.card.expiry_month),
                day=1
            )
            # A little trick to add a month
            date_expiry += timedelta(days=31)
            date_expiry -= timedelta(days=date_expiry.day - 1)

            if date_now >= date_expiry:
                self.__set_validation_error('Card expired')

    def __set_validation_error(self, message):
        raise ValueError(message)



# -*- coding: utf-8 -*-
from yookassa.domain.common.request_object import RequestObject
from yookassa.domain.models.amount import Amount
from yookassa.domain.models.deal import RefundDealData
from yookassa.domain.models.receipt import Receipt
from yookassa.domain.models.refund_source import RefundSource


class RefundRequest(RequestObject):
    """
    Class representing response object.

    Contains data
    """
    __payment_id = None

    __amount = None

    __description = None

    __receipt = None

    __sources = []

    __deal = None

    @property
    def payment_id(self):
        return self.__payment_id

    @payment_id.setter
    def payment_id(self, value):
        cast_value = str(value)
        if len(cast_value) == 36:
            self.__payment_id = cast_value
        else:
            raise ValueError('Invalid payment id value')

    @property
    def amount(self):
        return self.__amount

    @amount.setter
    def amount(self, value):
        if isinstance(value, dict):
            self.__amount = Amount(value)
        elif isinstance(value, Amount):
            self.__amount = value
        else:
            raise TypeError('Invalid amount value type')

    @property
    def description(self):
        return self.__description

    @description.setter
    def description(self, value):
        cast_value = str(value)
        if cast_value and len(cast_value) < 256:
            self.__description = cast_value
        else:
            raise ValueError('Invalid commend value')

    @property
    def sources(self):
        return self.__sources

    @sources.setter
    def sources(self, value):
        if isinstance(value, list):
            self.__sources = [RefundSource(item) for item in value]
        elif value is None:
            self.__sources = []
        else:
            raise TypeError('Invalid sources data type in refund_request.sources')

    @property
    def receipt(self):
        return self.__receipt

    @receipt.setter
    def receipt(self, value):
        if isinstance(value, dict):
            self.__receipt = Receipt(value)
        elif isinstance(value, Receipt):
            self.__receipt = value
        else:
            raise TypeError('Invalid receipt value type')

    @property
    def deal(self):
        return self.__deal

    @deal.setter
    def deal(self, value):
        if isinstance(value, dict):
            self.__deal = RefundDealData(value)
        elif isinstance(value, RefundDealData):
            self.__deal = value
        else:
            raise TypeError('Invalid deal value type')

    def validate(self):
        if not self.payment_id:
            self.__set_validation_error('Payment id not specified')

        if not self.amount:
            self.__set_validation_error('Amount not specified')

        if self.amount.value <= 0.0:
            self.__set_validation_error('Invalid amount value: ' + str(self.amount.value))

        if self.receipt and self.receipt.has_items():
            email = self.receipt.email
            phone = self.receipt.phone
            if not email and not phone:
                self.__set_validation_error('Both email and phone values are empty in receipt')

            if not self.receipt.tax_system_code and any(not item.vat_code for item in self.receipt.items):
                self.__set_validation_error('Item vat_id and receipt tax_system_id not specified')

    def __set_validation_error(self, message):
        raise ValueError(message)



# -*- coding: utf-8 -*-
import datetime
import re

from yookassa.domain.common import BaseObject


class Airline(BaseObject):
    """
    Class representing airline addendum data
    """
    __booking_reference = None

    __ticket_number = None

    __passengers = None

    __legs = None

    @property
    def booking_reference(self):
        return self.__booking_reference

    @booking_reference.setter
    def booking_reference(self, value):
        cast_value = str(value)
        if cast_value and len(cast_value) <= 20:
            self.__booking_reference = cast_value
        else:
            raise ValueError('Invalid booking_reference value')

    @property
    def ticket_number(self):
        return self.__ticket_number

    @ticket_number.setter
    def ticket_number(self, value):
        cast_value = str(value)
        if re.match('^[0-9]{1,150}$', cast_value):
            self.__ticket_number = cast_value
        else:
            raise ValueError('Invalid ticket_number value')

    @property
    def passengers(self):
        return self.__passengers

    @passengers.setter
    def passengers(self, value):
        if isinstance(value, list):
            passengers = []
            for passengerData in value:
                if isinstance(passengerData, dict):
                    passengers.append(Passenger(passengerData))
                elif isinstance(passengerData, Passenger):
                    passengers.append(passengerData)
                else:
                    raise TypeError('Invalid passengers data type in airline.passengers')

            self.__passengers = passengers
        else:
            raise TypeError('Invalid passengers value type in airline')

    @property
    def legs(self):
        return self.__legs

    @legs.setter
    def legs(self, value):
        if isinstance(value, list):
            legs = []
            for legData in value:
                if isinstance(legData, dict):
                    legs.append(Leg(legData))
                elif isinstance(legData, Leg):
                    legs.append(legData)
                else:
                    raise TypeError('Invalid legs data type in airline.passengers')

            self.__legs = legs
        else:
            raise TypeError('Invalid legs value type in airline')


class Passenger(BaseObject):
    __first_name = None

    __last_name = None

    @property
    def first_name(self):
        return self.__first_name

    @first_name.setter
    def first_name(self, value):
        cast_value = str(value)
        if cast_value and len(cast_value) <= 64:
            self.__first_name = cast_value
        else:
            raise ValueError('Invalid passengers first_name value')

    @property
    def last_name(self):
        return self.__last_name

    @last_name.setter
    def last_name(self, value):
        cast_value = str(value)
        if cast_value and len(cast_value) <= 64:
            self.__last_name = cast_value
        else:
            raise ValueError('Invalid passengers last_name value')


class Leg(BaseObject):
    __departure_airport = None

    __destination_airport = None

    __departure_date = None

    @property
    def departure_airport(self):
        return self.__departure_airport

    @departure_airport.setter
    def departure_airport(self, value):
        if re.match('^[A-Z]{3}$', value):
            self.__departure_airport = value
        else:
            raise ValueError('Invalid departure_airport value')

    @property
    def destination_airport(self):
        return self.__destination_airport

    @destination_airport.setter
    def destination_airport(self, value):
        if re.match('^[A-Z]{3}$', value):
            self.__destination_airport = value
        else:
            raise ValueError('Invalid destination_airport value')

    @property
    def departure_date(self):
        return self.__departure_date

    @departure_date.setter
    def departure_date(self, value):
        try:
            datetime.datetime.strptime(value, '%Y-%m-%d')
        except ValueError:
            raise ValueError("Incorrect data format, should be YYYY-MM-DD")

        self.__departure_date = value



# -*- coding: utf-8 -*-
from yookassa.domain.common.confirmation_type import ConfirmationType
from yookassa.domain.models.confirmation.confirmation import Confirmation


class ConfirmationMobileApplication(Confirmation):
    """
    Class representing mobile_application confirmation data object
    """
    __confirmation_url = None

    def __init__(self, *args, **kwargs):
        super(ConfirmationMobileApplication, self).__init__(*args, **kwargs)
        if self.type is None or self.type is not ConfirmationType.MOBILE_APPLICATION:
            self.type = ConfirmationType.MOBILE_APPLICATION

    @property
    def confirmation_url(self):
        return self.__confirmation_url

    @confirmation_url.setter
    def confirmation_url(self, value):
        self.__confirmation_url = value



# -*- coding: utf-8 -*-
import uuid

from yookassa.client import ApiClient
from yookassa.domain.common.http_verb import HttpVerb
from yookassa.domain.request.refund_request import RefundRequest
from yookassa.domain.response.refund_list_response import RefundListResponse
from yookassa.domain.response.refund_response import RefundResponse


class Refund:
    base_path = '/refunds'

    def __init__(self):
        self.client = ApiClient()

    @classmethod
    async def create(cls, params, idempotency_key=None):
        """
        Create refund

        :param params: data passed to API
        :param idempotency_key:
        :return:
        """
        instance = cls()
        path = cls.base_path
        if not idempotency_key:
            idempotency_key = uuid.uuid4()
        headers = {
            'Idempotence-Key': str(idempotency_key)
        }

        if isinstance(params, dict):
            params_object = RefundRequest(params)
        elif isinstance(params, RefundRequest):
            params_object = params
        else:
            raise TypeError('Invalid params value type')

        response = await instance.client.request(HttpVerb.POST, path, None, headers, params_object)
        return RefundResponse(response)

    @classmethod
    async def find_one(cls, refund_id):
        """
        Get refund information

        :param refund_id:
        :return: RefundResponse
        """
        instance = cls()
        if not isinstance(refund_id, str) or not refund_id:
            raise ValueError('Invalid payment_id value')
        path = instance.base_path + '/' + refund_id
        response = await instance.client.request(HttpVerb.GET, path)
        return RefundResponse(response)

    @classmethod
    async def list(cls, params):
        instance = cls()
        path = cls.base_path

        response = await instance.client.request(HttpVerb.GET, path, params)
        return RefundListResponse(response)



# -*- coding: utf-8 -*-
from yookassa.domain.common.type_factory import TypeFactory
from yookassa.domain.models.confirmation.confirmation_class_map import ConfirmationClassMap


class ConfirmationFactory(TypeFactory):
    def __init__(self):
        super(ConfirmationFactory, self).__init__(ConfirmationClassMap())



# -*- coding: utf-8 -*-


class WebhookNotificationType:
    """
    """
    NOTIFICATION = 'notification'


class WebhookNotificationEventType:
    """
    """
    PAYMENT_WAITING_FOR_CAPTURE = 'payment.waiting_for_capture'
    PAYMENT_SUCCEEDED = 'payment.succeeded'
    PAYMENT_CANCELED = 'payment.canceled'
    REFUND_SUCCEEDED = 'refund.succeeded'

    DEAL_CLOSED = 'deal.closed'
    PAYOUT_CANCELED = 'payout.canceled'
    PAYOUT_SUCCEEDED = 'payout.succeeded'



# -*- coding: utf-8 -*-
import uuid

from yookassa.client import ApiClient
from yookassa.domain.common import HttpVerb
from yookassa.domain.request import CapturePaymentRequest, PaymentRequest
from yookassa.domain.response import PaymentListResponse, PaymentResponse


class Payment:
    base_path = '/payments'

    def __init__(self):
        self.client = ApiClient()

    @classmethod
    async def find_one(cls, payment_id):
        """
        Get information about payment

        :param payment_id:
        :return: PaymentResponse
        """
        instance = cls()
        if not isinstance(payment_id, str) or not payment_id:
            raise ValueError('Invalid payment_id value')

        path = instance.base_path + '/' + payment_id
        response = await instance.client.request(HttpVerb.GET, path)
        return PaymentResponse(response)

    @classmethod
    async def create(cls, params, idempotency_key=None):
        """
        Create payment

        :param params: data passed to API
        :param idempotency_key:
        :return: PaymentResponse
        """
        instance = cls()
        path = cls.base_path

        if not idempotency_key:
            idempotency_key = uuid.uuid4()

        headers = {
            'Idempotence-Key': str(idempotency_key)
        }

        if isinstance(params, dict):
            params_object = PaymentRequest(params)
        elif isinstance(params, PaymentRequest):
            params_object = params
        else:
            raise TypeError('Invalid params value type')

        response = await instance.client.request(HttpVerb.POST, path, None, headers, params_object)
        return PaymentResponse(response)

    @classmethod
    async def capture(cls, payment_id, params=None, idempotency_key=None):
        """
        Capture payment

        :param payment_id:
        :param params: data passed to capture payment
        :param idempotency_key:
        :return: PaymentResponse
        """
        instance = cls()
        if not isinstance(payment_id, str) or not payment_id:
            raise ValueError('Invalid payment_id value')

        path = instance.base_path + '/' + payment_id + '/capture'

        if not idempotency_key:
            idempotency_key = uuid.uuid4()

        headers = {
            'Idempotence-Key': str(idempotency_key)
        }

        if isinstance(params, dict):
            params_object = CapturePaymentRequest(params)
        elif isinstance(params, CapturePaymentRequest):
            params_object = params
        else:
            params_object = None

        response = await instance.client.request(HttpVerb.POST, path, None, headers, params_object)
        return PaymentResponse(response)

    @classmethod
    async def cancel(cls, payment_id, idempotency_key=None):
        """
        Cancel payment

        :param payment_id:
        :param idempotency_key:
        :return: PaymentResponse
        """
        instance = cls()
        if not isinstance(payment_id, str) or not payment_id:
            raise ValueError('Invalid payment_id value')

        if not idempotency_key:
            idempotency_key = uuid.uuid4()

        path = instance.base_path + '/' + payment_id + '/cancel'
        headers = {
            'Idempotence-Key': str(idempotency_key)
        }
        response = await instance.client.request(HttpVerb.POST, path, None, headers)
        return PaymentResponse(response)

    @classmethod
    async def list(cls, params):
        instance = cls()
        path = cls.base_path

        response = await instance.client.request(HttpVerb.GET, path, params)
        return PaymentListResponse(response)



# -*- coding: utf-8 -*-


class Currency(object):
    """
    Class representing currency enum
    """
    RUB = "RUB"
    USD = "USD"
    EUR = "EUR"
    BYN = "BYN"
    CNY = "CNY"
    KZT = "KZT"
    UAH = "UAH"



# -*- coding: utf-8 -*-
from yookassa.domain.common.payment_method_type import PaymentMethodType
from yookassa.domain.models.payment_data.payment_data import ResponsePaymentData


class PaymentDataInstallments(ResponsePaymentData):
    def __init__(self, *args, **kwargs):
        super(PaymentDataInstallments, self).__init__(*args, **kwargs)
        if self.type is None or self.type is not PaymentMethodType.INSTALMENTS:
            self.type = PaymentMethodType.INSTALMENTS



# -*- coding: utf-8 -*-


class PaymentMethodType:
    """
    Constants representing payment_method_data values. Available values are:

    * yookassa.domain.common.PaymentMethodType.YOO_MONEY
    * yookassa.domain.common.PaymentMethodType.BANK_CARD
    * yookassa.domain.common.PaymentMethodType.SBERBANK
    * yookassa.domain.common.PaymentMethodType.CASH
    * yookassa.domain.common.PaymentMethodType.MOBILE_BALANCE
    * yookassa.domain.common.PaymentMethodType.PSB
    * yookassa.domain.common.PaymentMethodType.QIWI
    * yookassa.domain.common.PaymentMethodType.WEBMONEY
    * yookassa.domain.common.PaymentMethodType.ALFABANK
    * yookassa.domain.common.PaymentMethodType.APPLEPAY
    * yookassa.domain.common.PaymentMethodType.GOOGLE_PAY
    * yookassa.domain.common.PaymentMethodType.INSTALMENTS
    * yookassa.domain.common.PaymentMethodType.B2B_SBERBANK
    * yookassa.domain.common.PaymentMethodType.TINKOFF_BANK
    * yookassa.domain.common.PaymentMethodType.WECHAT
    * yookassa.domain.common.PaymentMethodType.SBP
    """
    YOO_MONEY = 'yoo_money'
    BANK_CARD = 'bank_card'
    SBERBANK = 'sberbank'
    CASH = 'cash'
    MOBILE_BALANCE = 'mobile_balance'
    PSB = 'psb'
    QIWI = 'qiwi'
    WEBMONEY = 'webmoney'
    ALFABANK = 'alfabank'
    APPLEPAY = 'apple_pay'
    GOOGLE_PAY = 'google_pay'
    INSTALMENTS = 'installments'
    B2B_SBERBANK = 'b2b_sberbank'
    TINKOFF_BANK = 'tinkoff_bank'
    WECHAT = 'wechat'
    SBP = 'sbp'



# -*- coding: utf-8 -*-
import typing
import httpx
import time
from requests.auth import _basic_auth_str

from yookassa import Configuration
from yookassa.domain.common import RequestObject, UserAgent
from yookassa.domain.exceptions import ApiError, BadRequestError, ForbiddenError, NotFoundError, \
    ResponseProcessingError, TooManyRequestsError, UnauthorizedError


class RetryTransport(httpx.HTTPTransport):
    """ Адаптация urllib3.Retry для HTTPX """

    def __init__(self, *args, total: int = 3, backoff_factor: float = 1, method_whitelist: typing.Optional[list] = None, status_forcelist: typing.Optional[list] = None, **kwargs):
        super(RetryTransport, self).__init__(*args, **kwargs)
        self.total = total
        self.backoff_factor = backoff_factor
        self.method_whitelist = method_whitelist
        self.status_forcelist = status_forcelist

    def handle_request(
            self,
            request: httpx.Request,
    ) -> httpx.Response:
        retry = 0
        resp = None
        retry_active = not self.method_whitelist or request.method in self.method_whitelist
        while retry < self.total:
            retry += 1
            if retry > 2:
                time.sleep(self.backoff_factor)
            try:
                if resp is not None:
                    resp.close()
                resp = super().handle_request(request)
            except Exception:
                if not retry_active:
                    raise
                continue
            if self.status_forcelist and resp.status_code in self.status_forcelist:
                continue
            break
        return resp


class ApiClient:
    endpoint = Configuration.api_endpoint()

    def __init__(self):
        self.configuration = Configuration.instantiate()
        self.shop_id = self.configuration.account_id
        self.shop_password = self.configuration.secret_key
        self.auth_token = self.configuration.auth_token
        self.timeout = self.configuration.timeout
        self.max_attempts = self.configuration.max_attempts

        self.user_agent = UserAgent()
        if self.configuration.agent_framework:
            self.user_agent.framework = self.configuration.agent_framework
        if self.configuration.agent_cms:
            self.user_agent.cms = self.configuration.agent_cms
        if self.configuration.agent_module:
            self.user_agent.module = self.configuration.agent_module

    async def request(self, method="", path="", query_params=None, headers=None, body=None):
        if isinstance(body, RequestObject):
            body.validate()
            body = dict(body)

        request_headers = self.prepare_request_headers(headers)
        raw_response = await self.execute(body, method, path, query_params, request_headers)

        if raw_response.status_code != 200:
            self.__handle_error(raw_response)

        return raw_response.json()

    async def execute(self, body, method, path, query_params, request_headers):
        session = self.get_session()
        raw_response = session.request(method,
                                       f'{self.endpoint}{path}',
                                       params=query_params,
                                       headers=request_headers,
                                       json=body,
                                       timeout=None)
        return raw_response

    def get_session(self) -> httpx.Client:
        session = httpx.Client(
            timeout=httpx.Timeout(self.timeout / 1000, connect=self.timeout / 1000),
            transport=RetryTransport()
        )
        return session

    def prepare_request_headers(self, headers):
        request_headers = {'Content-type': 'application/json'}
        if self.auth_token is not None:
            auth_headers = {"Authorization": "Bearer " + self.auth_token}
        else:
            auth_headers = {"Authorization": _basic_auth_str(self.shop_id, self.shop_password)}

        request_headers.update(auth_headers)

        request_headers.update({"YM-User-Agent": self.user_agent.get_header_string()})

        if isinstance(headers, dict):
            request_headers.update(headers)
        return request_headers

    def __handle_error(self, raw_response):
        http_code = raw_response.status_code
        if http_code == BadRequestError.HTTP_CODE:
            raise BadRequestError(raw_response.json())
        elif http_code == ForbiddenError.HTTP_CODE:
            raise ForbiddenError(raw_response.json())
        elif http_code == NotFoundError.HTTP_CODE:
            raise NotFoundError(raw_response.json())
        elif http_code == TooManyRequestsError.HTTP_CODE:
            raise TooManyRequestsError(raw_response.json())
        elif http_code == UnauthorizedError.HTTP_CODE:
            raise UnauthorizedError(raw_response.json())
        elif http_code == ResponseProcessingError.HTTP_CODE:
            raise ResponseProcessingError(raw_response.json())
        else:
            raise ApiError(raw_response.text)



# -*- coding: utf-8 -*-
from yookassa.domain.request.capture_payment_request import CapturePaymentRequest


class CapturePaymentBuilder(object):

    def __init__(self):
        self.__request = CapturePaymentRequest()

    def set_amount(self, value):
        self.__request.amount = value
        return self

    def set_receipt(self, value):
        self.__request.receipt = value
        return self

    def set_airline(self, value):
        self.__request.airline = value
        return self

    def set_metadata(self, value):
        self.__request.metadata = value
        return self

    def set_transfers(self, value):
        self.__request.transfers = value
        return self

    def build(self):
        return self.__request



# -*- coding: utf-8 -*-
from decimal import Decimal

from yookassa.domain.common import BaseObject
from yookassa.domain.models.amount import Amount
from yookassa.domain.models.receipt_item_supplier import ReceiptItemSupplier


class ReceiptItemRequest(BaseObject):
    """
    Class representing receipt item data wrapper object

    Used in Receipt
    """
    __description = None

    __quantity = None

    __amount = None

    __vat_code = None

    __payment_mode = None

    __payment_subject = None

    __product_code = None

    __country_of_origin_code = None

    __customs_declaration_number = None

    __excise = None

    __supplier = None

    __agent_type = None

    @property
    def description(self):
        return self.__description

    @description.setter
    def description(self, value):
        self.__description = str(value)

    @property
    def quantity(self):
        """
        :return Decimal:
        """
        return self.__quantity

    @quantity.setter
    def quantity(self, value):
        self.__quantity = Decimal(str(float(value)))

    @property
    def amount(self):
        return self.__amount

    @amount.setter
    def amount(self, value):
        if isinstance(value, dict):
            self.__amount = Amount(value)
        elif isinstance(value, Amount):
            self.__amount = value
        else:
            raise TypeError('Invalid amount value type')

    @property
    def vat_code(self):
        return self.__vat_code

    @vat_code.setter
    def vat_code(self, value):
        self.__vat_code = int(value)

    @property
    def payment_mode(self):
        return self.__payment_mode

    @payment_mode.setter
    def payment_mode(self, value):
        self.__payment_mode = str(value)

    @property
    def payment_subject(self):
        return self.__payment_subject

    @payment_subject.setter
    def payment_subject(self, value):
        self.__payment_subject = str(value)

    @property
    def product_code(self):
        return self.__product_code

    @product_code.setter
    def product_code(self, value):
        self.__product_code = str(value)

    @property
    def country_of_origin_code(self):
        return self.__country_of_origin_code

    @country_of_origin_code.setter
    def country_of_origin_code(self, value):
        self.__country_of_origin_code = str(value)

    @property
    def customs_declaration_number(self):
        return self.__customs_declaration_number

    @customs_declaration_number.setter
    def customs_declaration_number(self, value):
        self.__customs_declaration_number = str(value)

    @property
    def excise(self):
        """
        :return Decimal:
        """
        return self.__excise

    @excise.setter
    def excise(self, value):
        self.__excise = Decimal(str(float(value)))

    @property
    def supplier(self):
        return self.__supplier

    @supplier.setter
    def supplier(self, value):
        if isinstance(value, dict):
            self.__supplier = ReceiptItemSupplier(value)
        elif isinstance(value, ReceiptItemSupplier):
            self.__supplier = value
        else:
            raise TypeError('Invalid supplier value type')

    @property
    def agent_type(self):
        return self.__agent_type

    @agent_type.setter
    def agent_type(self, value):
        self.__agent_type = str(value)



# -*- coding: utf-8 -*-
from yookassa.domain.request import DealRequest


class DealRequestBuilder(object):
    def __init__(self):
        self.__request = DealRequest()

    def set_type(self, value):
        self.__request.type = value
        return self

    def set_fee_moment(self, value):
        self.__request.fee_moment = value
        return self

    def set_description(self, value):
        self.__request.description = value
        return self

    def set_metadata(self, value):
        self.__request.metadata = value
        return self

    def build(self):
        return self.__request



# -*- coding: utf-8 -*-
from decimal import Decimal, ROUND_HALF_UP

from yookassa.domain.common import BaseObject


class Amount(BaseObject):
    """
    Class representing amount data wrapper object
    """
    __value = None

    __currency = None

    @property
    def value(self):
        """
        :return Decimal:
        """
        return self.__value

    @value.setter
    def value(self, value):
        self.__value = Decimal(str(float(value))).quantize(Decimal('1.11'), rounding=ROUND_HALF_UP)

    @property
    def currency(self):
        return self.__currency

    @currency.setter
    def currency(self, value):
        self.__currency = str(value)



# -*- coding: utf-8 -*-
from yookassa.domain.request.refund_request import RefundRequest


class RefundRequestBuilder(object):
    def __init__(self):
        self.__request = RefundRequest()

    def set_payment_id(self, value):
        self.__request.payment_id = value
        return self

    def set_amount(self, value):
        self.__request.amount = value
        return self

    def set_description(self, value):
        self.__request.description = value
        return self

    def set_receipt(self, value):
        self.__request.receipt = value
        return self

    def set_sources(self, value):
        self.__request.sources = value
        return self

    def set_deal(self, value):
        self.__request.deal = value
        return self

    def build(self):
        return self.__request



# -*- coding: utf-8 -*-
from yookassa.domain.common import BaseObject
from yookassa.domain.models.amount import Amount

DESCRIPTION_MAX_LENGTH = 128


class Transfer(BaseObject):
    """
    Class representing payment transfer wrapper object

    Used in Payment
    """
    __account_id = None

    __amount = None

    __platform_fee_amount = None

    __description = None

    __metadata = None

    @property
    def account_id(self):
        return self.__account_id

    @account_id.setter
    def account_id(self, value):
        self.__account_id = str(value)

    @property
    def amount(self):
        return self.__amount

    @amount.setter
    def amount(self, value):
        if isinstance(value, dict):
            self.__amount = Amount(value)
        elif isinstance(value, Amount):
            self.__amount = value
        else:
            raise TypeError('Invalid transfer.amount value type')

    @property
    def platform_fee_amount(self):
        return self.__platform_fee_amount

    @platform_fee_amount.setter
    def platform_fee_amount(self, value):
        if isinstance(value, dict):
            self.__platform_fee_amount = Amount(value)
        elif isinstance(value, Amount):
            self.__platform_fee_amount = value
        else:
            raise TypeError('Invalid transfer.platform_fee_amount value type')

    @property
    def description(self):
        return self.__description

    @description.setter
    def description(self, value):
        cast_value = str(value)
        if cast_value:
            if len(cast_value) <= DESCRIPTION_MAX_LENGTH:
                self.__description = cast_value
            else:
                raise ValueError('The value of the description parameter is too long. Max length is {}'.format(
                    DESCRIPTION_MAX_LENGTH))
        else:
            raise ValueError('Invalid description value')

    @property
    def metadata(self):
        return self.__metadata

    @metadata.setter
    def metadata(self, value):
        if type(value) is dict:
            self.__metadata = value



# -*- coding: utf-8 -*-
import re

from yookassa.domain.common.payment_method_type import PaymentMethodType
from yookassa.domain.models.payment_data.response.payment_data_bank_card import PaymentDataBankCard


class PaymentDataSberbank(PaymentDataBankCard):
    __phone = None

    def __init__(self, *args, **kwargs):
        super(PaymentDataSberbank, self).__init__(*args, **kwargs)
        if self.type is None or self.type is not PaymentMethodType.SBERBANK:
            self.type = PaymentMethodType.SBERBANK

    @property
    def phone(self):
        return self.__phone

    @phone.setter
    def phone(self, value):
        cast_value = str(value)
        if re.match('^[0-9]{4,15}$', cast_value):
            self.__phone = cast_value
        else:
            raise ValueError('Invalid phone value type')



# -*- coding: utf-8 -*-
from yookassa.domain.common import BaseObject
from yookassa.domain.common.payment_method_type import PaymentMethodType
from yookassa.domain.models.amount import Amount
from yookassa.domain.models.payment_data.payment_data import PaymentData


class PaymentDataB2bSberbank(PaymentData):
    __payment_purpose = None

    __vat_data = None

    def __init__(self, *args, **kwargs):
        super(PaymentDataB2bSberbank, self).__init__(*args, **kwargs)
        if self.type is None or self.type is not PaymentMethodType.B2B_SBERBANK:
            self.type = PaymentMethodType.B2B_SBERBANK

    @property
    def payment_purpose(self):
        return self.__payment_purpose

    @payment_purpose.setter
    def payment_purpose(self, value):
        self.__payment_purpose = str(value)

    @property
    def vat_data(self):
        return self.__vat_data

    @vat_data.setter
    def vat_data(self, value):
        if isinstance(value, dict):
            self.__vat_data = VatData(value)
        elif isinstance(value, VatData):
            self.__vat_data = value
        else:
            raise TypeError('Invalid vat_data value type')


class VatData(BaseObject):
    __type = None

    __rate = None

    __amount = None

    @property
    def type(self):
        return self.__type

    @type.setter
    def type(self, value):
        if value in VatDataType.__dict__.values():
            self.__type = str(value)
        else:
            raise ValueError('Invalid type value')

    @property
    def rate(self):
        return self.__rate

    @rate.setter
    def rate(self, value):
        if value in VatDataRate.__dict__.values():
            self.__rate = int(value)
        else:
            raise ValueError('Invalid rate value')

    @property
    def amount(self):
        return self.__amount

    @amount.setter
    def amount(self, value):
        if isinstance(value, dict):
            self.__amount = Amount(value)
        elif isinstance(value, Amount):
            self.__amount = value
        else:
            raise TypeError('Invalid amount value type')


class VatDataType:
    CALCULATED = 'calculated'
    MIXED = 'mixed'
    UNTAXED = 'untaxed'


class VatDataRate:
    """
        Class representing rates available types enum
    """
    RATE_7 = 7
    RATE_10 = 10
    RATE_18 = 18
    RATE_20 = 20



# -*- coding: utf-8 -*-
from yookassa.domain.common import BaseObject
from yookassa.domain.common.data_context import DataContext


class ResponseObject(BaseObject):
    """
    Base class for request objects
    """
    @staticmethod
    def context():
        return DataContext.RESPONSE



# -*- coding: utf-8 -*-
from yookassa.domain.common import BaseObject
from yookassa.domain.models.amount import Amount


class TransferResponse(BaseObject):
    """
    Class representing payment transfer wrapper object

    Used in Payment
    """
    __account_id = None

    __amount = None

    __platform_fee_amount = None

    __status = None

    __description = None

    __metadata = None

    @property
    def account_id(self):
        return self.__account_id

    @account_id.setter
    def account_id(self, value):
        self.__account_id = str(value)

    @property
    def amount(self):
        return self.__amount

    @amount.setter
    def amount(self, value):
        if isinstance(value, dict):
            self.__amount = Amount(value)
        elif isinstance(value, Amount):
            self.__amount = value
        else:
            raise TypeError('Invalid amount value type')

    @property
    def platform_fee_amount(self):
        return self.__platform_fee_amount

    @platform_fee_amount.setter
    def platform_fee_amount(self, value):
        if isinstance(value, dict):
            self.__platform_fee_amount = Amount(value)
        elif isinstance(value, Amount):
            self.__platform_fee_amount = value
        else:
            raise TypeError('Invalid platform_fee_amount value type')

    @property
    def status(self):
        return self.__status

    @status.setter
    def status(self, value):
        self.__status = str(value)

    @property
    def description(self):
        return self.__description

    @description.setter
    def description(self, value):
        self.__description = str(value)

    @property
    def metadata(self):
        return self.__metadata

    @metadata.setter
    def metadata(self, value):
        if type(value) is dict:
            self.__metadata = value


class TransferStatus(object):
    """
    Class representing transfer.status values enum
    """
    PENDING = 'pending'
    WAITING_FOR_CAPTURE = 'waiting_for_capture'
    SUCCEEDED = 'succeeded'
    CANCELED = 'canceled'



# -*- coding: utf-8 -*-
from yookassa.domain.models.confirmation.confirmation import Confirmation


class ConfirmationRequest(Confirmation):
    """
    Base class confirmation request data objects
    """
    __locale = None

    @property
    def locale(self):
        return self.__locale

    @locale.setter
    def locale(self, value):
        self.__locale = str(value)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['yookassa',
 'yookassa.domain',
 'yookassa.domain.common',
 'yookassa.domain.exceptions',
 'yookassa.domain.models',
 'yookassa.domain.models.confirmation',
 'yookassa.domain.models.confirmation.request',
 'yookassa.domain.models.confirmation.response',
 'yookassa.domain.models.payment_data',
 'yookassa.domain.models.payment_data.request',
 'yookassa.domain.models.payment_data.response',
 'yookassa.domain.models.payout_data',
 'yookassa.domain.models.payout_data.request',
 'yookassa.domain.models.payout_data.response',
 'yookassa.domain.notification',
 'yookassa.domain.request',
 'yookassa.domain.response']

package_data = \
{'': ['*']}

install_requires = \
['deprecated>=1.2.13,<2.0.0',
 'distro>=1.8.0,<2.0.0',
 'httpx>=0.24.0,<0.25.0',
 'netaddr>=0.8.0,<0.9.0',
 'requests>=2.28.2,<3.0.0']

setup_kwargs = {
    'name': 'yookassa-async',
    'version': '0.1.1',
    'description': '',
    'long_description': "# YooKassa ASync API Python Client Library\n\n[![Build Status](https://travis-ci.org/yoomoney/yookassa-sdk-python.svg?branch=master)](https://travis-ci.org/yoomoney/yookassa-sdk-python)\n[![Latest Stable Version](https://img.shields.io/pypi/v/yookassa.svg)](https://pypi.org/project/yookassa/)\n[![Total Downloads](https://img.shields.io/pypi/dm/yookassa.svg)](https://pypi.org/project/yookassa/)\n[![License](https://img.shields.io/pypi/l/yookassa.svg)](https://git.yoomoney.ru/projects/SDK/repos/yookassa-sdk-python)\n\nRussian | [English](README.en.md)\n\nКлиент для работы с платежами по [API ЮKassa](https://yookassa.ru/developers/api)\nПодходит тем, у кого способ подключения к ЮKassa называется API.\n\nАсинхронная версия.\n\n## Требования\n\n1. Python >=3.7\n2. pip\n\n## Установка\n### C помощью pip\n\n1. Установите pip.\n2. В консоли выполните команду\n```bash\npip install --upgrade yookassa\n```\n\n### С помощью easy_install\n1. Установите easy_install.\n2. В консоли выполните команду\n```bash\neasy_install --upgrade yookassa\n```\n\n### Вручную\n\n1. В консоли выполните команды:\n```bash\nwget https://pypi.python.org/packages/5a/be/5eafdfb14aa6f32107e9feb6514ca1ad3fe56f8e5ee59d20693b32f7e79f/yookassa-1.0.0.tar.gz#md5=46595279b5578fd82a199bfd4cd51db2\ntar zxf yookassa-1.0.0.tar.gz\ncd yookassa-1.0.0\npython setup.py install\n```\n\n## Начало работы\n\n1. Импортируйте модуль\n```python\nimport yookassa\n```\n2. Установите данные для конфигурации\n```python\nfrom yookassa import Configuration\n\nConfiguration.configure('<Идентификатор магазина>', '<Секретный ключ>')\n```\n\nили\n\n```python\nfrom yookassa import Configuration\n\nConfiguration.account_id = '<Идентификатор магазина>'\nConfiguration.secret_key = '<Секретный ключ>'\n```\n\nили через oauth\n\n```python\nfrom yookassa import Configuration\n\nConfiguration.configure_auth_token('<Oauth Token>')\n```\n\nЕсли вы согласны участвовать в развитии SDK, вы можете передать данные о вашем фреймворке, cms или модуле:\n```python\nfrom yookassa import Configuration\nfrom yookassa.domain.common.user_agent import Version\n\nConfiguration.configure('<Идентификатор магазина>', '<Секретный ключ>')\nConfiguration.configure_user_agent(\n    framework=Version('Django', '2.2.3'),\n    cms=Version('Wagtail', '2.6.2'),\n    module=Version('Y.CMS', '0.0.1')\n)\n```\n\n3. Вызовите нужный метод API. [Подробнее в документации к API ЮKassa](https://yookassa.ru/developers/api)\n\n## Примеры использования SDK\n\n#### [Настройки SDK API ЮKassa](./docs/examples/01-configuration.md)\n* [Аутентификация](./docs/examples/01-configuration.md#Аутентификация)\n* [Статистические данные об используемом окружении](./docs/examples/01-configuration.md#Статистические-данные-об-используемом-окружении)\n* [Получение информации о магазине](./docs/examples/01-configuration.md#Получение-информации-о-магазине)\n* [Работа с Webhook](./docs/examples/01-configuration.md#Работа-с-Webhook)\n* [Входящие уведомления](./docs/examples/01-configuration.md#Входящие-уведомления)\n\n#### [Работа с платежами](./docs/examples/02-payments.md)\n* [Запрос на создание платежа](./docs/examples/02-payments.md#Запрос-на-создание-платежа)\n* [Запрос на создание платежа через билдер](./docs/examples/02-payments.md#Запрос-на-создание-платежа-через-билдер)\n* [Запрос на частичное подтверждение платежа](./docs/examples/02-payments.md#Запрос-на-частичное-подтверждение-платежа)\n* [Запрос на отмену незавершенного платежа](./docs/examples/02-payments.md#Запрос-на-отмену-незавершенного-платежа)\n* [Получить информацию о платеже](./docs/examples/02-payments.md#Получить-информацию-о-платеже)\n* [Получить список платежей с фильтрацией](./docs/examples/02-payments.md#Получить-список-платежей-с-фильтрацией)\n\n#### [Работа с возвратами](./docs/examples/03-refunds.md)\n* [Запрос на создание возврата](./docs/examples/03-refunds.md#Запрос-на-создание-возврата)\n* [Запрос на создание возврата через билдер](./docs/examples/03-refunds.md#Запрос-на-создание-возврата-через-билдер)\n* [Получить информацию о возврате](./docs/examples/03-refunds.md#Получить-информацию-о-возврате)\n* [Получить список возвратов с фильтрацией](./docs/examples/03-refunds.md#Получить-список-возвратов-с-фильтрацией)\n\n#### [Работа с чеками](./docs/examples/04-receipts.md)\n* [Запрос на создание чека](./docs/examples/04-receipts.md#Запрос-на-создание-чека)\n* [Запрос на создание чека через билдер](./docs/examples/04-receipts.md#Запрос-на-создание-чека-через-билдер)\n* [Получить информацию о чеке](./docs/examples/04-receipts.md#Получить-информацию-о-чеке)\n* [Получить список чеков с фильтрацией](./docs/examples/04-receipts.md#Получить-список-чеков-с-фильтрацией)\n",
    'author': 'Илья Снимщиков',
    'author_email': 'igsnimschikov@edu.hse.ru',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-

from yookassa.domain.common import BaseObject
from yookassa.domain.common.data_context import DataContext


class RequestObject(BaseObject):
    """
    Base class for request objects
    """
    @staticmethod
    def context():
        return DataContext.REQUEST

    def validate(self):
        """
        Validate request data
        """
        pass



# -*- coding: utf-8 -*-
from yookassa.domain.common.receipt_type import ReceiptType
from yookassa.domain.common.request_object import RequestObject
from yookassa.domain.models.receipt_customer import ReceiptCustomer
from yookassa.domain.models.settlement import Settlement
from yookassa.domain.request.receipt_item_request import ReceiptItemRequest


class ReceiptRequest(RequestObject):

    __type = None

    __send = None

    __customer = None

    __tax_system_code = None

    __items = []

    __settlements = []

    __payment_id = None

    __refund_id = None

    __on_behalf_of = None

    @property
    def type(self):
        return self.__type

    @type.setter
    def type(self, value):
        self.__type = str(value)

    @property
    def send(self):
        return self.__send

    @send.setter
    def send(self, value):
        if isinstance(value, bool):
            self.__send = value
        else:
            raise TypeError('Invalid send value type in receipt_request')

    @property
    def customer(self):
        return self.__customer

    @customer.setter
    def customer(self, value):
        if isinstance(value, dict):
            self.__customer = ReceiptCustomer(value)
        elif isinstance(value, ReceiptCustomer):
            self.__customer = value
        else:
            raise TypeError('Invalid customer value type in receipt_request')

    @property
    def email(self):
        return self.__customer.email if self.__customer is not None else None

    @email.setter
    def email(self, value):
        if self.__customer is None:
            self.__customer = ReceiptCustomer()
        self.__customer.email = str(value)

    @property
    def phone(self):
        return self.__customer.phone if self.__customer is not None else None

    @phone.setter
    def phone(self, value):
        if self.__customer is None:
            self.__customer = ReceiptCustomer()
        self.__customer.phone = str(value)

    @property
    def tax_system_code(self):
        return self.__tax_system_code

    @tax_system_code.setter
    def tax_system_code(self, value):
        if isinstance(value, int):
            self.__tax_system_code = value
        else:
            raise TypeError('Invalid tax_system_code value type in receipt_request')

    @property
    def items(self):
        return self.__items

    @items.setter
    def items(self, value):
        if isinstance(value, list):
            items = []
            for item in value:
                if isinstance(item, dict):
                    items.append(ReceiptItemRequest(item))
                elif isinstance(item, ReceiptItemRequest):
                    items.append(item)
                else:
                    raise TypeError('Invalid item type in receipt.items')
            self.__items = items
        elif value is None:
            self.__items = []
        else:
            raise TypeError('Invalid items value type in receipt_request')

    @property
    def settlements(self):
        return self.__settlements

    @settlements.setter
    def settlements(self, value):
        if isinstance(value, list):
            items = []
            for item in value:
                if isinstance(item, dict):
                    items.append(Settlement(item))
                elif isinstance(item, Settlement):
                    items.append(item)
                else:
                    raise TypeError('Invalid settlement type in receipt.settlements')
            self.__settlements = items
        elif value is None:
            self.__settlements = []
        else:
            raise TypeError('Invalid settlements value type in receipt_request')

    @property
    def payment_id(self):
        return self.__payment_id

    @payment_id.setter
    def payment_id(self, value):
        self.__refund_id = None
        self.__payment_id = str(value)

    @property
    def refund_id(self):
        return self.__refund_id

    @refund_id.setter
    def refund_id(self, value):
        self.__payment_id = None
        self.__refund_id = str(value)

    @property
    def on_behalf_of(self):
        return self.__on_behalf_of

    @on_behalf_of.setter
    def on_behalf_of(self, value):
        self.__on_behalf_of = str(value)

    def validate(self):
        if self.type is None:
            self.__set_validation_error('Receipt type not specified')

        if self.send is None:
            self.__set_validation_error('Receipt send not specified')

        if self.customer is not None:
            email = self.customer.email
            phone = self.customer.phone
            if not email and not phone:
                self.__set_validation_error('Both email and phone values are empty in customer')
        else:
            self.__set_validation_error('Receipt customer not specified')

        if not self.has_items():
            self.__set_validation_error('Receipt items not specified')

        if not self.has_settlements():
            self.__set_validation_error('Receipt settlements not specified')

        if self.type is ReceiptType.PAYMENT and self.payment_id is None:
            self.__set_validation_error('Receipt payment_id not specified')

        if self.type is ReceiptType.REFUND and self.refund_id is None and self.payment_id is None:
            self.__set_validation_error('Receipt refund_id or payment_id not specified')

    def has_items(self):
        return bool(self.items)

    def has_settlements(self):
        return bool(self.settlements)

    def __set_validation_error(self, message):
        raise ValueError(message)



# -*- coding: utf-8 -*-
from yookassa.domain.request import PayoutRequest


class PayoutRequestBuilder(object):
    def __init__(self):
        self.__request = PayoutRequest()

    def set_amount(self, value):
        self.__request.amount = value
        return self

    def set_description(self, value):
        self.__request.description = value
        return self

    def set_payout_token(self, value):
        self.__request.payout_token = value
        return self

    def set_payout_destination_data(self, value):
        self.__request.payout_destination_data = value
        return self

    def set_deal(self, value):
        self.__request.deal = value
        return self

    def set_metadata(self, value):
        self.__request.metadata = value
        return self

    def build(self):
        return self.__request



# -*- coding: utf-8 -*-
from decimal import Decimal

from yookassa.domain.common import BaseObject
from yookassa.domain.models.amount import Amount


class ReceiptItem(BaseObject):
    """
    Class representing receipt item data wrapper object

    Used in Receipt
    """
    __description = None

    __quantity = None

    __amount = None

    __vat_code = None

    __payment_mode = None

    __payment_subject = None

    __product_code = None

    __country_of_origin_code = None

    __customs_declaration_number = None

    __excise = None

    @property
    def description(self):
        return self.__description

    @description.setter
    def description(self, value):
        self.__description = str(value)

    @property
    def quantity(self):
        """
        :return Decimal:
        """
        return self.__quantity

    @quantity.setter
    def quantity(self, value):
        self.__quantity = Decimal(str(float(value)))

    @property
    def amount(self):
        return self.__amount

    @amount.setter
    def amount(self, value):
        if isinstance(value, dict):
            self.__amount = Amount(value)
        elif isinstance(value, Amount):
            self.__amount = value
        else:
            raise TypeError('Invalid amount value type')

    @property
    def vat_code(self):
        return self.__vat_code

    @vat_code.setter
    def vat_code(self, value):
        self.__vat_code = int(value)

    @property
    def payment_mode(self):
        return self.__payment_mode

    @payment_mode.setter
    def payment_mode(self, value):
        self.__payment_mode = str(value)

    @property
    def payment_subject(self):
        return self.__payment_subject

    @payment_subject.setter
    def payment_subject(self, value):
        self.__payment_subject = str(value)

    @property
    def product_code(self):
        return self.__product_code

    @product_code.setter
    def product_code(self, value):
        self.__product_code = str(value)

    @property
    def country_of_origin_code(self):
        return self.__country_of_origin_code

    @country_of_origin_code.setter
    def country_of_origin_code(self, value):
        self.__country_of_origin_code = str(value)

    @property
    def customs_declaration_number(self):
        return self.__customs_declaration_number

    @customs_declaration_number.setter
    def customs_declaration_number(self, value):
        self.__customs_declaration_number = str(value)

    @property
    def excise(self):
        """
        :return Decimal:
        """
        return self.__excise

    @excise.setter
    def excise(self, value):
        self.__excise = Decimal(str(value))


class PaymentMode(object):
    """
    Class representing payment_mode values enum
    """
    FULL_PREPAYMENT = 'full_prepayment'
    PARTIAL_PREPAYMENT = 'partial_prepayment'
    ADVANCE = 'advance'
    FULL_PAYMENT = 'full_payment'
    PARTIAL_PAYMENT = 'partial_payment'
    CREDIT = 'credit'
    CREDIT_PAYMENT = 'credit_payment'


class PaymentSubject(object):
    """
    Class representing payment_subject values enum
    """
    COMMODITY = 'commodity'
    EXCISE = 'excise'
    JOB = 'job'
    SERVICE = 'service'
    GAMBLING_BET = 'gambling_bet'
    GAMBLING_PRIZE = 'gambling_prize'
    LOTTERY = 'lottery'
    LOTTERY_PRIZE = 'lottery_prize'
    INTELLECTUAL_ACTIVITY = 'intellectual_activity'
    PAYMENT = 'payment'
    AGENT_COMMISSION = 'agent_commission'
    COMPOSITE = 'composite'
    ANOTHER = 'another'
